#!/usr/bin/env python
# -*- encoding: utf-8; py-indent-offset: 4 -*-

#
# 10.07.2017 : Th.L. : Cisco PSIRT API Framework
# 16.05.2018 : Th.L. : fixed handling if state changes form found to notfound (delete old psirt found file)
#
#  https://developer.cisco.com/docs/support-apis/
#
import logging
import os
import json
import time
import ConfigParser
import ciscosupport
import ciscoapi
import sys


def psirt_remove_id_file(psirt_path, psirt_id):
    # set logg modul name <file>:<module>.<function>
    logger = logging.getLogger(__file__ + ':' + __name__ + '.' + sys._getframe().f_code.co_name)

    # delete psirt file
    try:
        logger.debug('delete psirt id file : %s' % psirt_path + psirt_id)
        os.remove(psirt_path + psirt_id)
    except OSError:
        pass


def psirt_dump_record(psirt_record, psirt_id, psirt_path, psirt_path_request):
    # set logg modul name <file>:<module>.<function>
    logger = logging.getLogger(__file__ + ':' + __name__ + '.' + sys._getframe().f_code.co_name)

    with open(psirt_path + psirt_id, 'w') as f:
        json.dump(psirt_record, f)
    # delete request file
    psirt_remove_id_file(psirt_path_request, psirt_id)

    return


def check_psirt_record(psirt_record, psirt_id, psirt_path_found, psirt_path_request):
    # set logg modul name <file>:<module>.<function>
    logger = logging.getLogger(__file__ + ':' + __name__ + '.' + sys._getframe().f_code.co_name)

    for advisory in psirt_record.get('advisories'):
        # remove unwanted information from advisories
        advisory.pop('productNames', None)
        advisory.pop('ipsSignatures', None)
        advisory.pop('iosRelease', None)
        advisory.pop('cvrfUrl', None)
        advisory.pop('ovalUrl', None)
        advisory.pop('summary', None)
        for key in advisory.keys():
            if advisory.get(key, None) in [['NA'], 'NA']:
                advisory.pop(key, None)

    psirt_dump_record(psirt_record, psirt_id, psirt_path_found, psirt_path_request)

    return

def main():

    def get_psirt_id_list(product_family):
        # create list of ID's to request PSIRT status for
        psirt_id_list = ciscosupport.get_ids_from_dir(path_request + product_family)
        logger.debug('psirt requests : %s' % psirt_id_list)
        # remove already found ID's from list
        psirt_id_list = ciscosupport.remove_ids_from_list(psirt_id_list, path_found + product_family)
        logger.debug('psirt requests : %s' % psirt_id_list)
        # remove not found ID's from list
        psirt_id_list = ciscosupport.remove_ids_from_list(psirt_id_list, path_not_found + product_family)
        logger.debug('psirt requests : %s' % psirt_id_list)

        # refresh psirt after 1 day by default
        psirt_id_list = ciscosupport.refresh_ids_from_dir(path_not_found + product_family, psirt_refresh_notfound, psirt_id_list, True)
        logger.debug('psirt requests : %s' % psirt_id_list)
        psirt_id_list = ciscosupport.refresh_ids_from_dir(path_found + product_family, psirt_refresh_found, psirt_id_list, False)
        logger.debug('psirt requests : %s' % psirt_id_list)

        return psirt_id_list

    def update_psirt_id(psirt_records, family_name):
        for psirt_record in psirt_records:
            if family_name in ['IOS', 'IOS-XE']:
                psirt_id = psirt_record.get('version')
            else:
                psirt_id = psirt_record.get('family')

            if psirt_record.get('advisories') != 'notfound':
                check_psirt_record(psirt_record, psirt_id, path_found + family_name + '/', path_request + family_name + '/')
            else:
                psirt_dump_record(psirt_record, psirt_id, path_not_found + family_name + '/', path_request + family_name + '/')
                # remove psirt_found file (happens when product family is removed form bug ID)
                psirt_remove_id_file(path_found + family_name + '/', psirt_id)
        return


    ciscosupport.set_logging('debug')
    # set logg modul name <file>:<module>.<function>
    logger = logging.getLogger(__file__ + ':' + __name__ + '.' + sys._getframe().f_code.co_name)

    conf_file = '~/etc/ciscoapi/ciscoapi.conf'
    conf_file = os.path.expanduser(conf_file)

    # default paths
    base_path = '~/var/ciscoapi'

    access_token = ''
    lifetime = 0

    # refresh time in days
    psirt_refresh_found = 1
    psirt_refresh_notfound = 1

    wait_after_start = True
    max_wait_time = 15
    loglevel = 'warning'
    # loglevel = 'debug'

    if os.path.isfile(conf_file):
        configParser = ConfigParser.RawConfigParser()
        configParser.read(conf_file)
        if configParser.has_option('global', 'base_path'):
            wait_after_start = configParser.getboolean('global', 'base_path')
        if configParser.has_option('global', 'wait_after_start'):
            wait_after_start = configParser.getboolean('global', 'wait_after_start')
        if configParser.has_option('global', 'max_wait_time'):
            max_wait_time = configParser.getint('global', 'max_wait_time')
        if configParser.has_option('global', 'loglevel'):
            loglevel = configParser.get('global', 'loglevel')
        if configParser.has_option('psirt', 'psirt_refresh_found'):
            psirt_refresh_found = configParser.getint('psirt', 'psirt_refresh_found')
        if configParser.has_option('psirt', 'psirt_refresh_notfound'):
            psirt_refresh_notfound = configParser.getint('psirt', 'psirt_refresh_notfound')
    else:
        logger.critical('Config file not found (%s).' % conf_file)
        return False

    ciscosupport.set_logging(loglevel)

    base_path = ciscosupport.expand_path(base_path)
    psirt_dir = base_path + 'psirt'
    path_found = psirt_dir + '/found/'
    path_not_found = psirt_dir + '/not_found/'
    path_request = psirt_dir + '/request/'

    psirt_ios = get_psirt_id_list('IOS')
    psirt_ios_xe = get_psirt_id_list('IOS-XE')
    psirt_family = get_psirt_id_list('family')

    if (psirt_ios == []) and psirt_ios_xe == [] and psirt_family == []:
        logger.debug('all list are empty. Do nothing.')
        return

    # wait random time after startup
    if wait_after_start:
        ciscosupport.sleep_random(max_wait_time)

    # get time since epoch in seconds
    starttime = int(time.time())

    # request access_token if no token available or if is expired
    if (access_token == '') or lifetime < starttime:
        response = ciscoapi.get_access_token()
        access_token = response.get('access_token', '')
        lifetime = int(response.get('lifetime', 3600))

    lifetime = starttime + lifetime - 30

    if access_token == '':
        logger.critical('failed to get access_token')
        return

    if psirt_family != []:
        psirt_records = ciscoapi.get_psirt_by_product_family(psirt_family, access_token)
        update_psirt_id(psirt_records, 'family')

    if psirt_ios_xe != []:
        psirt_records = ciscoapi.get_psirt_by_iosxe_version(psirt_ios_xe, access_token)
        update_psirt_id(psirt_records, 'IOS-XE')

    if psirt_ios != []:
        psirt_records = ciscoapi.get_psirt_by_ios_version(psirt_ios, access_token)
        update_psirt_id(psirt_records, 'IOS')

main()
