#!/usr/bin/env python
# -*- encoding: utf-8; py-indent-offset: 4 -*-

#
# 15.04.2017 : Th.L. : Cisco SN2INFO API Framework
#
# https://developer.cisco.com/docs/support-apis/#serial-number-to-information
#
import logging
import os
import json
import time
import ConfigParser
import ciscosupport
import ciscoapi
import sys


def sn2info_split_covered(sn2inforecords):
    # set logg modul name <file>:<module>.<function>
    logger = logging.getLogger(__file__ + ':' + __name__ + '.' + sys._getframe().f_code.co_name)

    sn2info_covered = []
    sn2info_notcovered = []

    for response in sn2inforecords:
        sn2inforecord = response.get('serial_numbers')
#        PaginationResponseRecord = response.get('PaginationResponseRecord')
#        PageIndex = PaginationResponseRecord.get('PageIndex')
#        LastIndex = PaginationResponseRecord.get('LastIndex')
#        PageRecords = PaginationResponseRecord.get('PageRecords')
#        TotalRecords = PaginationResponseRecord.get('TotalRecords')
        if sn2inforecord is not None:
            for serial in sn2inforecord:
                if serial.get('is_covered') == 'YES':
                    sn2info_covered.append(serial)
                    logger.debug('SN2INFO covered  : %s' % serial.get('sr_no'))
                else:
                    sn2info_notcovered.append(serial)
                    logger.debug('SN2INFO not covered : %s' % serial.get('sr_no'))
    return {'sn2info_covered': sn2info_covered, 'sn2info_notcovered': sn2info_notcovered}


def sn2info_save_serials(sn2infos, path):
    # set logg modul name <file>:<module>.<function>
    logger = logging.getLogger(__file__ + ':' + __name__ + '.' + sys._getframe().f_code.co_name)

    saved = []

    for sn2info in sn2infos:
        serial = str(sn2info.get('sr_no'))
        if serial != '':
            with open(path + serial, 'w') as f:
                json.dump(sn2info, f)
            saved.append(serial)

    return saved


def main():

    ciscosupport.set_logging('debug')
    # set logg modul name <file>:<module>.<function>
    logger = logging.getLogger(__file__ + ':' + __name__ + '.' + sys._getframe().f_code.co_name)

    conf_file = '~/etc/ciscoapi/ciscoapi.conf'
    conf_file = os.path.expanduser(conf_file)

    # default paths
    base_path = '~/var/ciscoapi'
    
    access_token = ''
    lifetime = 0
    sn2info = []

    # refresh time in days
    sn2info_refresh_covered = 31
    sn2info_refresh_notcovered = 7

    wait_after_start = True
    max_wait_time = 15
    loglevel = 'warning'

    if os.path.isfile(conf_file):
        configParser = ConfigParser.RawConfigParser()
        configParser.read(conf_file)
        if configParser.has_option('global', 'base_path'):
            wait_after_start = configParser.getboolean('global', 'base_path')
        if configParser.has_option('sn2info', 'sn2info_refresh_covered'):
            sn2info_refresh_covered = configParser.getint('sn2info', 'sn2info_refresh_covered')
        if configParser.has_option('sn2info', 'sn2info_refresh_notcovered'):
            sn2info_refresh_notcovered = configParser.getint('sn2info', 'sn2info_refresh_notcovered')

        if configParser.has_option('global', 'wait_after_start'):
            wait_after_start = configParser.getboolean('global', 'wait_after_start')
        if configParser.has_option('global', 'max_wait_time'):
            max_wait_time = configParser.getint('global', 'max_wait_time')
        if configParser.has_option('global', 'loglevel'):
            loglevel = configParser.get('global', 'loglevel')
    else:
        logger.critical('Config file not found (%s).' % conf_file)
        return False

    ciscosupport.set_logging(loglevel)

    sn2info_dir = base_path + '/sn2info'
    path_found = ciscosupport.expand_path(sn2info_dir + '/found/')
    path_not_found = ciscosupport.expand_path(sn2info_dir + '/not_found/')
    path_request = ciscosupport.expand_path(sn2info_dir + '/request/')

    # create list of serial numbers to request SN2INFO status for
    sn2info = ciscosupport.get_ids_from_dir(path_request)
    logger.debug('sn2info requests : %s' % sn2info)
    # remove covered serials from list
    sn2info = ciscosupport.remove_ids_from_list(sn2info, path_found)
    logger.debug('sn2info requests : %s' % sn2info)
    # remove not covered serials from list
    sn2info = ciscosupport.remove_ids_from_list(sn2info, path_not_found)
    logger.debug('sn2info requests : %s' % sn2info)

    # refresh sn2info serials after 31 days by default
    sn2info = ciscosupport.refresh_ids_from_dir(path_not_found, sn2info_refresh_notcovered, sn2info, True)
    logger.debug('sn2info requests : %s' % sn2info)
    sn2info = ciscosupport.refresh_ids_from_dir(path_found, sn2info_refresh_covered, sn2info, False)
    logger.debug('sn2info requests : %s' % sn2info)

    if sn2info == [] :
        logger.debug('all list are empty. Do nothing.')
        return

    # wait random time after startup
    if wait_after_start:
        ciscosupport.sleep_random(max_wait_time)

    # get time since epoch in seconds
    starttime = int(time.time())

    # request access_token if no token available or if is expired
    if (access_token == '') or lifetime < starttime:
        response = ciscoapi.get_access_token()
        access_token = response.get('access_token', '')
        lifetime = int(response.get('lifetime', 3600))

    lifetime = starttime + lifetime - 30

    if access_token == '':
        logger.critical('failed to get access_token')
        return

    if sn2info != []:
        sn2inforecords = ciscoapi.get_coverage_summary_by_serials(sn2info, access_token)
        logger.debug('sn2info response: %s' % sn2inforecords)
        sn2info = sn2info_split_covered(sn2inforecords)
        serials = sn2info_save_serials(sn2info.get('sn2info_covered'), path_found)
        ciscosupport.remove_ids_from_dir(serials, path_request)
        serials = sn2info_save_serials(sn2info.get('sn2info_notcovered'), path_not_found)
        ciscosupport.remove_ids_from_dir(serials, path_request)
        # delete serials from covered were the status changed to uncovered
        ciscosupport.remove_ids_from_dir(serials, path_found)


main()

